/* include ---------------------------------------------------------*/
#include "main.h"


/* private define -------------------------------------------------*/
#define	DS18B20_DQ_SCK_APBxClock_FUN			RCC_APB2PeriphClockCmd
#define	DS18B20_DQ_GPIO_CLK						RCC_APB2Periph_GPIOA

#define	DS18B20_DQ_GPIO_PORT					GPIOA
#define	DS18B20_DQ_GPIO_PIN						GPIO_Pin_7

#define	DS18B20_DQ_0							GPIO_ResetBits(DS18B20_DQ_GPIO_PORT,DS18B20_DQ_GPIO_PIN )
#define	DS18B20_DQ_1							GPIO_SetBits(DS18B20_DQ_GPIO_PORT,DS18B20_DQ_GPIO_PIN )
#define	DS18B20_DQ_IN()							GPIO_ReadInputDataBit( DS18B20_DQ_GPIO_PORT,DS18B20_DQ_GPIO_PIN)

/* private variables ----------------------------------------------*/


/* private function prototypes ------------------------------------*/
static void DS18B20_Mode_IPU(void);
static void DS18B20_Mode_Out_PP(void);
static void DS18B20_Rst(void);
static uint8_t DS18B20_Presence(void);
static uint8_t DS18B20_ReadBit(void);
static uint8_t DS18B20_ReadByte(void);
static void DS18B20_WriteByte(uint8_t dat);
static void DS18B20_SkipRom(void);
static void DS18B20_MatchRom(void);

static uint8_t Init(void);
static void ReadId(uint8_t * ds18b20_id);
static float GetTemp_SkipRom(void);
static float GetTemp_MatchRom( uint8_t * ds18b20_id );

/* public variables -----------------------------------------------*/
DS18B20_t  DS18B20 = 
{
	Init,
	ReadId,
	GetTemp_SkipRom,
	GetTemp_MatchRom
};

/**
  * @brief  ʼ
  * @param  None
  * @retval None
  * @note   None
*/
static uint8_t Init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	DS18B20_DQ_SCK_APBxClock_FUN ( DS18B20_DQ_GPIO_CLK, ENABLE);
														   
  	GPIO_InitStructure.GPIO_Pin = DS18B20_DQ_GPIO_PIN;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  	GPIO_Init ( DS18B20_DQ_GPIO_PORT , &GPIO_InitStructure );

	DS18B20_DQ_1;
	
	DS18B20_Rst();
	
	return DS18B20_Presence ();
}
/**
  * @brief  ʹDS18B20-DATAűΪģʽ
  * @param  None
  * @retval None
  * @note   None
*/
static void DS18B20_Mode_IPU(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	GPIO_InitStructure.GPIO_Pin = DS18B20_DQ_GPIO_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;	

	GPIO_Init(DS18B20_DQ_GPIO_PORT, &GPIO_InitStructure);
}

/**
  * @brief  ʹDS18B20-DATAűΪģʽ
  * @param  None
  * @retval None
  * @note   None
*/
static void DS18B20_Mode_Out_PP(void)
{
 	GPIO_InitTypeDef GPIO_InitStructure;
															   
  	GPIO_InitStructure.GPIO_Pin = DS18B20_DQ_GPIO_PIN;	
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;     
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  	GPIO_Init(DS18B20_DQ_GPIO_PORT, &GPIO_InitStructure);
}

/**
  * @brief  ӻ͸λ
  * @param  None
  * @retval None
  * @note   None
*/
static void DS18B20_Rst(void)
{
	/* Ϊ */
	DS18B20_Mode_Out_PP();
	
	DS18B20_DQ_0;
	/* ٲ480usĵ͵ƽλź */
	DelayClass.DelayUs(750);
	
	/* ڲλźź轫 */
	DS18B20_DQ_1;
	
	/*ӻյĸλźź󣬻15~60usһ*/
	DelayClass.DelayUs(15);
}
/**
  * @brief  ӻصĴ
  * @param  None
  * @retval 0 ɹ 1 ʧ
  * @note   None
*/
static uint8_t DS18B20_Presence(void)
{
	uint8_t pulse_time = 0;
	
	/* Ϊ */
	DS18B20_Mode_IPU();
	
	/* ȴĵΪһ60~240usĵ͵ƽź 
	 * ûʱӻյĸλźź󣬻15~60usһ
	 */
	while( DS18B20_DQ_IN() && pulse_time<100 )
	{
		pulse_time++;
		DelayClass.DelayUs(1);
	}	
	/* 100us󣬴嶼ûе*/
	if( pulse_time >=100 )
		return 1;
	else
		pulse_time = 0;
	
	/* 嵽Ҵڵʱ䲻ܳ240us */
	while( !DS18B20_DQ_IN() && pulse_time<240 )
	{
		pulse_time++;
		DelayClass.DelayUs(1);
	}	
	if( pulse_time >=240 )
		return 1;
	else
		return 0;
}

/**
  * @brief  DS18B20ȡһbit
  * @param  None
  * @retval None
  * @note   None
*/
static uint8_t DS18B20_ReadBit(void)
{
	uint8_t dat;
	
	/* 0Ͷ1ʱҪ60us */	
	DS18B20_Mode_Out_PP();
	/* ʱʼ >1us <15us ĵ͵ƽź */
	DS18B20_DQ_0;
	DelayClass.DelayUs(10);
	/* ó룬ͷߣⲿ轫 */
	DS18B20_Mode_IPU();
	if( DS18B20_DQ_IN() == SET )
	{
		dat = 1;
	}
	else
	{
		dat = 0;
	}
	/* ʱοʱͼ */
	DelayClass.DelayUs(45);
	
	return dat;
}

/**
  * @brief  DS18B20һֽڣλ
  * @param  None
  * @retval None
  * @note   None
*/
static uint8_t DS18B20_ReadByte(void)
{
	uint8_t i, j, dat = 0;	
	
	for(i=0; i<8; i++) 
	{
		j = DS18B20_ReadBit();		
		dat = (dat) | (j<<i);
	}
	
	return dat;
}
/**
  * @brief  дһֽڵDS18B20λ
  * @param  None
  * @retval None
  * @note   None
*/
static void DS18B20_WriteByte(uint8_t dat)
{
	uint8_t i, testb;
	DS18B20_Mode_Out_PP();
	
	for( i=0; i<8; i++ )
	{
		testb = dat&0x01;
		dat = dat>>1;		
		/* д0д1ʱҪ60us */
		if (testb)
		{
			DS18B20_DQ_0;
			/* 1us < ʱ < 15us */
			DelayClass.DelayUs(8);
			
			DS18B20_DQ_1;
			DelayClass.DelayUs(58);
			
		}		
		else
		{
			DS18B20_DQ_0;
			/* 60us < Tx 0 < 120us */
			DelayClass.DelayUs(70);
			
			DS18B20_DQ_1;			
			/* 1us < Trec(ָʱ) < */
			DelayClass.DelayUs(2);
		}
	}
}

/**
  * @brief  ƥ DS18B20 ROM
  * @param  None
  * @retval None
  * @note   None
*/
static void DS18B20_SkipRom(void)
{
	DS18B20_Rst();	   
	
	DS18B20_Presence();	 
	
	DS18B20_WriteByte(0XCC);/*  ROM */
	
}

/**
  * @brief  ִƥ DS18B20 ROM
  * @param  None
  * @retval None
  * @note   None
*/
static void DS18B20_MatchRom(void)
{
	DS18B20_Rst();	   
	
	DS18B20_Presence();	 
	
	DS18B20_WriteByte(0X55);		/* ƥ ROM */
	
}
/**
  * @brief  ƥ ROM »ȡ DS18B20 ¶ֵ 
  * @param  None
  * @retval None
  * @note   None
*/
static float GetTemp_SkipRom(void)
{
	uint8_t tpmsb, tplsb;
	short s_tem;
	float f_tem;
	
	DS18B20_SkipRom ();
	DS18B20_WriteByte(0X44);				/* ʼת */
	
	DS18B20_SkipRom ();
	DS18B20_WriteByte(0XBE);				/* ¶ֵ */
	
	tplsb = DS18B20_ReadByte();		 
	tpmsb = DS18B20_ReadByte(); 
	
	s_tem = tpmsb<<8;
	s_tem = s_tem | tplsb;
	
	if( s_tem < 0 )		/* ¶ */
	{
		f_tem = (~s_tem+1) * 0.0625;
	}	
	else
	{
		f_tem = s_tem * 0.0625;
	}
	return f_tem;
}

/**
  * @brief  ƥ ROM »ȡ DS18B20 ID
  * @param  ds18b20_id ڴ DS18B20 кŵ׵ַ
  * @retval None 
  * @note   None
*/
static void ReadId(uint8_t * ds18b20_id)
{
	uint8_t uc;
	
	DS18B20_WriteByte(0x33);       //ȡк

	for ( uc = 0; uc < 8; uc ++ )
	{
		ds18b20_id [ uc ] = DS18B20_ReadByte();
	}
}

/**
  * @brief  ƥ ROM »ȡ DS18B20 ¶ֵ
  * @param  ds18b20_id  DS18B20 кŵ׵ַ
  * @retval None
  * @note   None
*/
static float GetTemp_MatchRom( uint8_t * ds18b20_id )
{
	uint8_t tpmsb, tplsb, i;
	short s_tem;
	float f_tem;
	
	DS18B20_MatchRom ();            //ƥROM
	
	for(i=0;i<8;i++)
	{
		DS18B20_WriteByte ( ds18b20_id [i]);	
	}
	
	DS18B20_WriteByte(0X44);				/* ʼת */

	DS18B20_MatchRom ();            //ƥROM
	
	for(i=0;i<8;i++)
	{
		DS18B20_WriteByte (ds18b20_id [i]);	
	}
	
	DS18B20_WriteByte(0XBE);				/* ¶ֵ */
	
	tplsb = DS18B20_ReadByte();		 
	tpmsb = DS18B20_ReadByte(); 
	
	
	s_tem = tpmsb<<8;
	s_tem = s_tem | tplsb;
	
	if( s_tem < 0 )		/* ¶ */
	{
		f_tem = (~s_tem+1) * 0.0625;	
	}
	else
	{
		f_tem = s_tem * 0.0625;
	}
	return f_tem; 	

}
